#include "snl.h"
#include "avcall.h"
#include "parser.h"
#include "packet.h"

#include "functions.h"

bool
Functions::call(int fd, void *buf, int len, bool answer) {
	string s, ret, fnc, par;
	char *ret_s, *ptr[256];
	int ret_i, i, idx = 0;
	bool eol = false;
	double f, ret_f;
	av_alist args;
	Function *fn;
	Packet pkt;

	if (pkt.build(buf, len)) return (true);
	// get type of return parameter, function name and parameter list
	if (pkt.type() == 's') pkt.pop(ret); else return (true);
	if (pkt.type() == 's') pkt.pop(fnc); else return (true);
	if (pkt.type() == 's') pkt.pop(par); else return (true);
	// check syntax of function call
	if (!(fn = func[fnc.c_str()])) return (true);
	if (fn->ret != ret)            return (true);
	if (fn->par != par)            return (true);
	// prepare function call
	switch (ret[0]) {
		case 'v': av_start_void   (args, fn->fn);                 break;
		case 'i': av_start_int    (args, fn->fn, &ret_i);         break;
		case 'f': av_start_double (args, fn->fn, &ret_f);         break;
		case 's': av_start_ptr    (args, fn->fn, char *, &ret_s); break;
		default: return (true);
	}
	// read parameter list
	while (!eol) {
		switch (pkt.type()) {
			case 'i': pkt.pop(i); av_int    (args, i); break;
			case 'f': pkt.pop(f); av_double (args, f); break;
			case 's':
				pkt.pop(s);
				ptr[idx] = strdup(s.c_str()); // XXX strdup() really needed?
				av_ptr(args, char *, ptr[idx++]);
			break;
			case 'v':             break;
			case '0': eol = true; break;
			default: return (true);
		}
	}
	// call function
	av_call(args);
	// build return packet
	if (answer) {
		pkt.reset();
		pkt.push("v");
		pkt.push("return");
		pkt.push(ret);
		switch (ret[0]) {
			case 'i': pkt.push(ret_i); break;
			case 'f': pkt.push(ret_f); break;
			case 's': pkt.push(ret_s); break;
		}
		pkt.type('0'); // end of list
		snl_write(fd, pkt.data(), pkt.size());
	}
	// cleanup and return
	for (int x=0; x<idx; x++) free(ptr[x]);
	return (false); // ok
}

bool
Functions::reg(const char *fmt, void *fn) {
	Function *f;

	f = new Function();
	f->fn = fn;
	if (!split(fmt, f->ret, f->fnc, f->par)) {
		// store function in hash map
		func[f->fnc.c_str()] = f;
		// keep f in a vector for deleting in destructor
		functions.push_back(f);
		return (false); // ok
	}
	return (true); // error
}

Functions::~Functions() {
	for (unsigned int i=0; i<functions.size(); i++) delete (functions[i]);
}
