#include <sys/time.h> // gettimeofday()
#include <signal.h>   // signal()
#include <unistd.h>   // usleep()
#include <stdlib.h>   // atoi(), malloc()
#include <string.h>   // memset()
#include <stdio.h>    // printf()

#include "odai.h"

static bool running = true;

static char *
ping(const char *data) {
	return ((char *)data);
}

static void
error(int err) {
	printf("libodai error %i, exiting\n", err);
	exit(err);
}

void
quit(int sig) {
	running = false;
}

int
main(int argc, char **argv) {
	int seq = 0, count = -1, size = 64, wait = 1000000;
	float cur, avg, sum = 0.0, min = 3.0, max = 0.0;
	unsigned short int port = 3000;
	struct timeval tv;
	char *src, *dst;
	long usec;

	for (int i=1; i<argc; i++) {
		if (!strcmp(argv[i], "-i")) wait  = atoi(argv[i+1]) * 1000000;
		if (!strcmp(argv[i], "-s")) size  = atoi(argv[i+1]);
		if (!strcmp(argv[i], "-c")) count = atoi(argv[i+1]);
		if (!strcmp(argv[i], "-p")) port  = atoi(argv[i+1]);
		if (!strcmp(argv[i], "-f")) wait  = 0;
		if (!strcmp(argv[i], "-h")) {
			printf("USAGE: oping [-f] [-p port] [-s size] [-i wait] [-c count]\n");
			exit(0);
		}
	}
	if (wait < 0) wait = 0;
	if (size < 1) size = 1;

	signal(SIGINT,  quit);
	signal(SIGQUIT, quit);
	signal(SIGHUP,  quit);

	Server s("localhost", port);
	s.reg("s ping(s)", (void *)ping);
	usleep(250000); // wait a little bit
	Client c("localhost", port);
	c.err(error);

	src = (char *)malloc(size);
	dst = (char *)malloc(size);
	if (!(src) || !(dst)) {
		printf("memory allocation error\n");
		exit(1);
	}
	memset(src, 'A', size);
	src[size-1] = '\0';

	printf("OPING localhost (127.0.0.1): %i bytes data\n", size);
	while (running) {
		gettimeofday(&tv, NULL);
		usec = tv.tv_usec;
		if (c.fnc(&dst, "ping(s)", src)) {
			printf("TIMEOUT\n");
		} else {
			gettimeofday(&tv, NULL);
			usec = tv.tv_usec - usec;
			if (usec < 0) usec += 1000000;
			cur = (float)usec/1000;
			if (cur < min) min = cur;
			if (cur > max) max = cur;
			sum += cur;
			printf("%i bytes: seq = %i time = %0.3f ms\n", size, seq, cur);
		}
		if (wait) usleep(wait);
		if (++seq == count) quit(0);
	}
	avg = sum/seq;
	printf("\n--- localhost oping statistics ---\n");
	printf("%i packets transmitted\n", seq);
	printf("round trip min/avg/max = %0.3f/%0.3f/%0.3f ms\n", min, avg, max);

	return (0);
}
