#include <stdlib.h>
#include <stdio.h>

#include "error.h"
#include "parser.h"

#include "packet.h"

void
Packet::append(const char *str) {
	buf += str;
	buf += '\0';
}

char
Packet::type() {
	string tmp;
	int i;

	pop(tmp);
	i = atoi(tmp.c_str());
	return ((char)i);
}

void
Packet::type(char type) {
	char buf[16];

	snprintf(buf, 15, "%i", (int)type);
	append(buf);
}

void
Packet::push(int i) {
	char buf[16];

	type('i');
	snprintf(buf, 15, "%i", i);
	append(buf);
}

void
Packet::push(char *s) {
	type('s');
	append(s);
}

void
Packet::push(string s) {
	type('s');
	append(s.c_str());
}

void
Packet::push(double f) {
	char buf[32];

	type('f');
	snprintf(buf, 31, "%f", f);
	append(buf);
}

void
Packet::push(vector<int> v) {
	if (v.size()) {
		type('I');
		push((int)v.size());
		for (unsigned int i=0; i<v.size(); i++) {
			push(v[i]);
		}
	}
}

void
Packet::push(vector<string> v) {
	if (v.size()) {
		type('S');
		push((int)v.size());
		for (unsigned int i=0; i<v.size(); i++) {
			push(v[i]);
		}
	}
}

void
Packet::push(vector<double> v) {
	if (v.size()) {
		type('F');
		push((int)v.size());
		for (unsigned int i=0; i<v.size(); i++) {
			push(v[i]);
		}
	}
}

void
Packet::pop(int &i) {
	string tmp;

	pop(tmp);
	i = atoi(tmp.c_str());
}

void
Packet::pop(string &s) {
	int p;

	p = buf.find('\0', pos);
	s = buf.substr(pos, p-pos);
	pos += p-pos+1;
}

void
Packet::pop(double &f) {
	string tmp;

	pop(tmp);
	f = atof(tmp.c_str());
}

void
Packet::pop(vector<int> &v) {
	int size, val;

	v.clear();
	pop(size);
	for (int i=0; i<size; i++) {
		pop(val);
		v.push_back(val);
	}
}

void
Packet::pop(vector<string> &v) {
	int size;
	string val;

	v.clear();
	pop(size);
	for (int i=0; i<size; i++) {
		pop(val);
		v.push_back(val);
	}
}

void
Packet::pop(vector<double> &v) {
	int size;
	double val;

	v.clear();
	pop(size);
	for (int i=0; i<size; i++) {
		pop(val);
		v.push_back(val);
	}
}

void
Packet::rewind() {
	pos = 0;
}

void
Packet::reset() {
	buf.resize(0);
	pos = 0;
}

void
Packet::assign(void *b, int s) {
	buf.assign((char *)b, s);
	pos = 0;
}

char *
Packet::data() {
	return ((char *)buf.data());
}

int
Packet::size() {
	return (buf.length());
}

bool
Packet::build(void *b, int s) {
	assign(b, s);
	return (false);
}

bool
Packet::build(const char *fmt, va_list args) {
	string ret, fnc, par;
	double f;
	char *s;
	int i;

	if (split(fmt, ret, fnc, par)) return (true);
	push(ret);
	push(fnc);
	push(par);
	for (int cur=0; cur<(int)par.size(); cur++) {
		switch (par[cur]) {
			case 'i': i = va_arg(args, int);    push(i); break;
			case 'f': f = va_arg(args, double); push(f); break;
			case 's': s = va_arg(args, char *); push(s); break;
			case 'v':                                    break;
			default:	return (true);
		}
	}
	type('0'); // end of list
	return (false);
}

Packet::Packet() {
	buf.resize(0);
	pos = 0;
}
