#include "semaphore.h"

/**
	Creates the semapgore with initially Value resources.
*/
Semaphore::Semaphore(int val) {
	current = val;
	pthread_mutex_init(&mutex, 0);
	pthread_cond_init(&condition, 0);
}

/**
	Destroy the Semaphore.
*/
Semaphore::~Semaphore() {
	pthread_mutex_destroy(&mutex);
	pthread_cond_destroy(&condition);
}

/**
	Tries to acquire a resource. If there are no resource to be got, then the
	caller will block until another thread does an up() and releases a resource.
*/
int Semaphore::down() {
	int cur;
  
	pthread_mutex_lock(&mutex);
	// loop until we get a value on the semaphore > 0
	while (current <= 0) {
		// wait for someone to up the semaphore
		pthread_cond_wait(&condition, &mutex);
	}
	// decrease the value and store it
	current--;
	cur = current;
	// we're ready to unlock the mutex
	pthread_mutex_unlock(&mutex);
	// return the value we had
	return (cur);
}

/**
	Releases a reource, letting a locked thread continue. The caller need not
	have acquired a down() previously, but this operation is quite useless
	otherwise.
*/
int Semaphore::up() {
	int cur;
  
	// lock the mutex
	pthread_mutex_lock(&mutex);
	// increase the semaphore value
	current++;
	cur = current;
	// we're ready to unlock the mutex
	pthread_mutex_unlock(&mutex);
	// send a signal letting a blocked thread continue (if any)
	pthread_cond_signal(&condition);
	// return the value we had
	return (cur);
}

/**
	Reads the value of the semaphore and returns it. This should not be used
	instead of down(), if you need to allcoated a resource, but only as a way
	to get a hint as to where we currently are going!
*/
int Semaphore::value () {
	int cur;
  
	// lock the mutex
	pthread_mutex_lock(&mutex);
	// get the value
	cur = current;
	// we're ready to unlock the mutex
	pthread_mutex_unlock(&mutex);
	// return the value we had
	return (cur);
}
