#include <stdarg.h>
#include <unistd.h>

#include "functions.h"
#include "packet.h"
#include "error.h"
#include "server.h"

SNL_READ_CALLBACK(Server::read_callback) {
	Server *server;

	server = (Server *)data;
	server->mutex.lock();
	if ((error == SNL_ERR_CLOSED) || (error == SNL_ERR_READ)) {
		server->clients.remove(fd);
	} else {
		server->functions.call(fd, buf, len);
	}
	server->mutex.unlock();
}

SNL_LISTEN_CALLBACK(Server::listen_callback) {
	Server *server;

	server = (Server *)data;
	server->clients.add(fd);
	snl_read(fd, read_callback, data);
}

Server::Server(const char *name, unsigned short port) {
	if (snl_listen(name, port, listen_callback, this)) {
		throw (ODAI_ERR_SOCKET);
	}
}

int
Server::reg(const char *fmt, void *fn) {
	return (functions.reg(fmt, fn));
}

int
Server::msg(const char *fmt, ...) {
	string str = "v ";
	va_list args;
	Packet pkt;
	int err;

	str += fmt;
	va_start(args, fmt);
	err = pkt.build(str.c_str(), args);
	va_end(args);
	if (err) return (ODAI_ERR_SYNTAX);
	//mutex.lock();
	clients.broadcast(pkt.data(), pkt.size());
	//mutex.unlock();
	return (ODAI_OK);
}
