#include <sys/time.h>
#include <unistd.h>
#include <errno.h>

#include "signal.h"

Signal::Signal(int to) {
	timeout = to;
	pthread_mutex_init(&mutex, NULL);
	pthread_cond_init(&cond,   NULL);
}

Signal::~Signal() {
	while (pthread_cond_destroy(&cond))   usleep(1); // EBUSY
	while (pthread_mutex_destroy(&mutex)) usleep(1); // EBUSY
}

void
Signal::grab() {
	if (pthread_mutex_lock(&mutex)) throw (-1);
}

void
Signal::release() {
	if (pthread_mutex_unlock(&mutex)) throw (-1);
}

void
Signal::emit() {
	pthread_cond_signal(&cond);
}

bool
Signal::wait(int to) {
	int seconds = timeout;
	struct timespec wait;
	struct timeval now;

	if (to != 0) seconds = to;
	if (seconds > 0) {
		if (gettimeofday(&now, NULL) < 0) throw (-1);
		wait.tv_sec  = now.tv_sec + seconds;
		wait.tv_nsec = now.tv_usec * 1000;
		again:
		switch (pthread_cond_timedwait(&cond, &mutex, &wait)) {
			case ETIMEDOUT: return (true);
			case EINTR:     goto again;
		}
	} else {
		pthread_cond_wait(&cond, &mutex);
	}
	return (false);
}
