/**
	\file
	\author  Clemens Kirchgatterer <clemens@thf.ath.cx>
	\date    2002-02-23
	\brief   easy signaling system for multi threaded programming

	this class can be used to send signals from one thread to another.
*/

#ifndef _SIGNAL_H_
#define _SIGNAL_H_

#include <pthread.h>


/// signal class definition
class Signal {

public:

	/// constructor; default timeout in seconds, (to < 1) = forever
	Signal(int to = 1);
	/// destructor
	~Signal();

	/// grab the condition signal mutex
	void grab();
	/// release the condition signal mutex
	void release();

	/// emit the signal
	void emit();
	/// wait for the signal to arrive; timeout in soconds, (to = 0) keep default
	bool wait(int to = 0);

private:

	int timeout;

	pthread_mutex_t mutex;
	pthread_cond_t  cond;

};

#endif
